import socket
import time
import getpass
import pydbus
import logging


def get_host_name():
    """获取当前用户名"""
    username = getpass.getuser()
    return username


def get_host_time():
    """获取本机时间
    return: time_1
    """
    time_1 = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
    return time_1


def get_host_ip():
    """
    查询本机ip地址
    :return: ip
    """
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(('8.8.8.8', 80))
        ip = s.getsockname()[0]
    except Exception:
        s.close()
        ip = "127.0.0.1"
    return ip


def logger_config(log_path, logging_name,level):
    # 获取logger对象,取名
    logger = logging.getLogger(logging_name)
    # 输出DEBUG及以上级别的信息，针对所有输出的第一层过滤
    logger.setLevel(level=logging.DEBUG)
    # 获取文件日志句柄并设置日志级别，第二层过滤

    #判断handler是否存在,避免重复写入日志文件
    if not logger.handlers:
        handler = logging.FileHandler(log_path, encoding='UTF-8')
        if level == "info":
            handler.setLevel(logging.INFO)
        elif level == "waring":
            handler.setLevel(logging.WARNING)
        else:
            handler.setLevel(logging.ERROR)
        # logger.removeFilter(handler)
        # 生成并设置文件日志格式
        formatter = logging.Formatter('%(asctime)s  [%(levelname)s] - %(message)s', "%Y-%m-%d %H:%M:%S")
        handler.setFormatter(formatter)
        logger.addHandler(handler)
    return logger


def send_log(i,dic):
        lv="warning"
        message=""
        if i == 0:
            message = dic.get("error")
            lv = "error"
        elif i == 1:
            message = dic.get("info")
            lv = "info"
        elif i == 3:
            message = "清空iptables Filter表规则"
            lv = "info"
        elif i == 4:
            message = "未清空iptables Filter表规则"
            lv = "warning"
        str_1 = "'time':'{}','hostname':'{}','ip':'{}','name':'iptables_control','lv':'{}','message':'{}'".format(
            get_host_time(), get_host_name(), get_host_ip(), lv, message)
        str_2 = '{' + str_1 + '}'
        logger = logger_config(log_path='/var/log/iptables.log', logging_name="端口管控", level=lv)
        if lv == "info":
            logger.info(message)
        elif lv == "waring":
            logger.warning(message)
        else:
            logger.error(message)
        try:
            bus = pydbus.SystemBus()
            change = bus.get("org.log.sys_transmit", "/org/log/sys_transmit")
            change.log_transmit(str_2)
        except Exception:
            pass

