#!/usr/bin/env python3
import os
from gi.repository import GLib
from pydbus import SystemBus
import sys
import iptc
import addlog
import re
if os.getuid() != 0:
    print("please use root user")
    sys.exit(1)
loop = GLib.MainLoop()

class iptablesService(object):
    """
        <node>
	  <interface name='com.kylin.ipportcontrol'>
				<method name='iptables_rules'>
                    <arg type='a{ss}' name='dic' direction='in'/>
                    <arg type='i' name='response' direction='out'/>
                </method>
                 <method name='iptables_clean'>
                     <arg type='i' name='response' direction='out'/>
                </method>
        </interface>
        </node>
    """

    def iptables_rules(self,dic):  #传入存储的list
        global match
        try:
            #异常判断
            list = ['chain', 'action', 'target','protocol']
            for l in list:
                if not l in dic.keys():
                    raise ValueError
            listtar = ['ACCEPT', 'DROP']
            if not dic['target'] in listtar:
                raise ValueError
            table = iptc.Table(iptc.Table.FILTER)
            if not table.is_chain(dic['chain']):
                raise ValueError
            chain = iptc.Chain(table, dic['chain'])
            rule = iptc.Rule()
            listpro = ['tcp', 'udp']
            if not dic['protocol'] in listpro:
                raise ValueError
            listact = ['insert', 'delete']
            if not dic['action'] in listact:
                raise ValueError
            rule.protocol = dic['protocol']
            match = iptc.Match(rule, dic['protocol'])
            if 'sport' in dic.keys():
                match.sport = dic['sport']
                ma = re.match(
                    "^[1-9]$|(^[1-9][0-9]$)|(^[1-9][0-9][0-9]$)|(^[1-9][0-9][0-9][0-9]$)|(^[1-6][0-5][0-5][0-3][0-5]$)",
                    dic['sport'])
                if ma == None:
                    raise ValueError
            if 'dport' in dic.keys():
                match.dport = dic['dport']
                ma = re.match(
                    "^[1-9]$|(^[1-9][0-9]$)|(^[1-9][0-9][0-9]$)|(^[1-9][0-9][0-9][0-9]$)|(^[1-6][0-5][0-5][0-3][0-5]$)",
                    dic['dport'])
                if ma == None:
                    raise ValueError
            if 'source' in dic.keys():
                rule.src = dic['source']
                p = re.compile(
                    '^((25[0-5]|2[0-4]\d|((1\d{2})|([1-9]?\d))).){3}(25[0-5]|2[0-4]\d|((1\d{2})|([1-9]?\d)))$')
                if not p.match(dic['source']):
                    raise ValueError
            if 'destination' in dic.keys():
                rule.dst = dic['destination']
                p = re.compile(
                    '^((25[0-5]|2[0-4]\d|((1\d{2})|([1-9]?\d))).){3}(25[0-5]|2[0-4]\d|((1\d{2})|([1-9]?\d)))$')
                if not p.match(dic['destination']):
                    raise ValueError
            rule.add_match(match)
            target = iptc.Target(rule, dic['target'])
            rule.target = target
            if dic['action'] == 'insert':
                chain.insert_rule(rule)
                if dic['target'] == "ACCEPT":
                    target = iptc.Target(rule, 'DROP')
                    rule.target = target
                    removed = True
                    while removed == True:
                        removed = False
                        for rules in chain.rules:
                            if rules == rule:
                                chain.delete_rule(rules)
                                removed = True
                                break
                else:
                    target = iptc.Target(rule, "ACCEPT")
                    rule.target = target
                    removed = True
                    while removed == True:
                        removed = False
                        for rules in chain.rules:
                            if rules == rule:
                                chain.delete_rule(rules)
                                removed = True
                                break
            elif dic['action'] == 'delete':
                removed = True
                while removed == True:
                    removed = False
                    for rules in chain.rules:
                        if rules == rule:
                            chain.delete_rule(rules)
                            removed = True
                            break
            table.commit()
            self.save(dic)
        except ValueError:
            dic['error']='输入参数错误'
            addlog.send_log(0,dic)
            return -1
        else:
            if dic['target']=='ACCEPT':
                tar='ACCEPT'
            else:
                tar="DROP"
            if dic['action'] == 'insert':
                act='添加 '
            else:
                act="删除 "
            dic['info'] =act+dic['chain']+"链:源ip:"+dic.get('source','未设置')+"-目的ip:"+dic.get('destination','未设置')+"-源端口:"+dic.get('sport','未设置')+"-目的端口:"+dic.get('dport','未设置')+"-"+tar
            addlog.send_log(1,dic)
            return 1

    def save(self,dic):
        try:
            inline = '-A '
            # for key in dic.keys():
            if 'chain' in dic.keys():
                inline += dic['chain']
            if 'source' in dic.keys():
                inline += ' -s ' + dic.get('source')
            if 'destination' in dic.keys():
                inline += ' -d ' + dic['destination']
            if 'protocol' in dic.keys():
                inline += ' -p ' + dic['protocol']
            if 'sport' in dic.keys():
                inline += ' --sport ' + dic['sport']
            if 'dport' in dic.keys():
                inline += ' --dport ' + dic['dport']
            if 'target' in dic.keys():
                inline += ' -j ' + dic['target']+"\n"
            file = "/etc/network/iptables.up.rules"
            if dic['action']=='insert':
                if dic['target'] == "ACCEPT":
                    inline1 = inline.replace("ACCEPT", "DROP")
                else:
                    inline1 = inline.replace("DROP", "ACCEPT")
                with open(file, "r", encoding="utf-8") as f:
                    searchline = '*filter\n'
                    lines = f.readlines()
                    count = len(lines)
                    j = 0
                    while (j < count - 1):
                        l = lines[j]
                        if l == inline1:
                            lines.remove(lines[j])
                            print(lines)
                            count = count - 1
                        else:
                            j = j + 1
                    i = lines.index(searchline)
                    lines.insert(i + 1, inline)
                    with open(file, "w", encoding="utf-8") as f1:
                        for l in lines:
                            f1.write(l)
                        f1.close()
                    f.close()
            elif dic['action']=='delete':
                with open(file, "r", encoding="utf-8") as f:
                    lines = f.readlines()
                    count = len(lines)
                    j = 0
                    while (j < count - 1):
                        l = lines[j]
                        if l == inline:
                            lines.remove(lines[j])
                            count = count - 1
                        else:
                            j = j + 1
                    with open(file, "w", encoding="utf-8") as f1:
                        for l in lines:
                            f1.write(l)
                        f1.close()
                    f.close()

        except IOError:
            return 0
        else:
            return 1


    def iptables_clean(self):  # 传入存储的list
        try:
            table = iptc.Table(iptc.Table.FILTER)
            input = iptc.Chain(table, "INPUT")
            input.zero_counters()
            input.flush()
            output = iptc.Chain(table, "OUTPUT")
            output.flush()
            forward = iptc.Chain(table, "FORWARD")
            forward.flush()

            ###保存配置文件
            file = "/etc/network/iptables.up.rules"
            with open(file, "r", encoding="utf-8") as f:
                lines = f.readlines()
                count = len(lines)
                j = 0
                while (j < count - 1):
                    l = lines[j]
                    if not l.count('*filter') and not l.count('COMMIT'):
                        lines.remove(lines[j])
                        count = count - 1
                    else:
                        j = j + 1
                with open(file, "w", encoding="utf-8") as f1:
                    for l in lines:
                        f1.write(l)
                    f1.close()
                f.close()
        except IOError:
            addlog.send_log(4,{})
            return 0
        else:
            addlog.send_log(3, {})
            return 1

bus = SystemBus()
bus.publish("com.kylin.iptablescontrol", iptablesService())
loop.run()
